package org.freecompany.util.multicaster;

/**
 * This is a generic abstract event multicaster based on the pattern of the
 * {@link java.awt.AWTEventMulticaster}.  It's benefits are zero cost for all single listeners
 * and near zero cost for a few more after that.  Classes which commonly have greater
 * than a few listeners may be better off using a standard collection with fast
 * iteration.
 */
public abstract class AbstractMulticaster< T> implements Multicaster< T> {

	protected T a, b;

	protected AbstractMulticaster() { }
	protected AbstractMulticaster( final T a, final T b) {
		setA( a);
		setB( b);
	}

	public T getA() {
		return a;
	}

	public T getB() {
		return b;
	}

	public void setA( T a) {
		this.a = a;
	}

	public void setB( T b) {
		this.b = b;
	}

	public static <T> T removeInternal( Multicaster< T> object, T nullListener, T old) {
		if ( old == object.getA()) return object.getB();
		if ( old == object.getB()) return object.getA();
		T aa = remove( nullListener, object.getA(), old);
		T bb = remove( nullListener, object.getB(), old);
		if ( aa == object.getA() && bb == object.getB()) return ( T) object;
		return add( object.getClass(), nullListener, aa, bb);
	}

	public static < T> T add( Class< ? extends Multicaster> clazz, T nullListener, T a, T b) {
		try {
			return add( clazz.newInstance(), nullListener, a, b);
		} catch ( IllegalAccessException e) {
			throw new RuntimeException( e);
		} catch ( InstantiationException e) {
			throw new RuntimeException( e);
		}
	}

	public static < T> T add( Multicaster object, T nullListener, T a, T b) {
		if ( a == nullListener || a == null) return b;
		if ( b == nullListener || b == null) return a;
		object.setA( a);
		object.setB( b);
		return ( T) object;
	}

	public static < T> T remove( T nullListener, T current, T old) {
		if ( current == old || current == null) return nullListener;
		else if ( current instanceof Multicaster) return removeInternal((( Multicaster< T>) current), nullListener, old);
		else return current;
	}
}
