package org.freecompany.util.multicaster;

import java.lang.reflect.InvocationHandler;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.Proxy;

import static java.lang.reflect.Proxy.newProxyInstance;
import static org.freecompany.util.multicaster.AbstractMulticaster.removeInternal;

public class DefaultMulticaster {

	public static InvocationHandler NULL = new InvocationHandler() {
		public Object invoke( final Object proxy, final Method method, final Object[] args) { return null;}
	};

	/**
	 * Uses the current classloader as the passed class may well be a core JVM interface, in which case
	 * the default classloader would not be able to see the {@link Multicaster} class.
	 */
	public static < T> T empty( Class< T> clazz) {
		return ( T) ( Multicaster) newProxyInstance( DefaultMulticaster.class.getClassLoader(), new Class[] { clazz, Multicaster.class}, NULL);
	}

	/**
	 * Uses the current classloader as the passed class may well be a core JVM interface, in which case
	 * the default classloader would not be able to see the {@link Multicaster} class.
	 */
	public static < T> T add( Class< T> clazz, T a, T b) {
		Multicaster real = ( Multicaster) newProxyInstance( DefaultMulticaster.class.getClassLoader(), new Class[] { clazz, Multicaster.class}, new Handler());
		return AbstractMulticaster.add( real, empty( clazz), a, b);
	}

	public static < T> T remove( Class< T> clazz, T a, T b) {
		return AbstractMulticaster.remove(( T) empty( clazz), a, b);
	}

	public static class Handler< T> extends AbstractMulticaster< T> implements InvocationHandler {
		public Object invoke( final Object proxy, final Method method, final Object[] args) {
			if ( method.getName().equals( "setA")) setA(( T) args[ 0]);
			else if ( method.getName().equals( "setB")) setB(( T) args[ 0]);
			else if ( method.getName().equals( "getA")) return getA();
			else if ( method.getName().equals( "getB")) return getB();
			else if ( method.getName().equals( "removeInternal")) removeInternal( this, ( T) args[ 0], ( T) args[ 1]);
			else {
				try {
					method.invoke( getA(), args);
					method.invoke( getB(), args);
				} catch ( InvocationTargetException e) {
					throw new RuntimeException( e);
				} catch ( IllegalAccessException e) {
					throw new RuntimeException( e);
				}
			}
			return null;
		}
	}
}
