package org.freecompany.util.multicaster;

import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Method;

import static java.lang.reflect.Proxy.newProxyInstance;
import static java.util.logging.Level.FINE;
import static org.freecompany.util.multicaster.NotificationMulticaster.logging;

public class InvocationLogger implements InvocationHandler {

	protected final LoggerNotification notifier;

	public InvocationLogger( final Class< ?> clazz) {
		notifier = new LoggerNotification( clazz.getName());
	}

	public Object invoke( final Object proxy, final Method method, final Object[] args) {
		if ( args.length == 0 && "toString".equals( method.getName())) return toString();

		final StringBuilder builder = new StringBuilder( "(");
		if ( args == null) builder.append( ")");
		else {
			builder.append( " ");
			for ( int x = 0; x < args.length; x++) builder.append( args[ x]).append( x == args.length - 1 ? ")" : ", ");
		}
		notifier.log( FINE, builder.toString(), method.getDeclaringClass().getName(), method.getName(), null);
		return null;
	}

	public static < T> T newInstance( Class< T> clazz) {
		return ( T) newProxyInstance( clazz.getClassLoader(), new Class[] { clazz}, new InvocationLogger( clazz));
	}
}
